/*
 * ThreadedPrimesFixed.java
 *
 * Version of ThreadedPrimes.java that fixes its defects.  Has each
 * thread keep a private count of the number of primes found,
 * combining the results only at the end.  Also, divides the work
 * between the threads more evenly (the threads test 4k+1 and 4k+3
 * instead of giving one thread all the low numbers and the other all
 * the high numbers).
 */

public class ThreadedPrimesFixed {

    static int sharedPCount;
    static Object lock = new Object();

    static class PrimeFinder implements Runnable {

	long from;
	long to;

	int pCount;

	public PrimeFinder(long From, long To) {
	    from = From;
	    to = To;
	    pCount = 0;
	}

	public void run() {
	    long nextCand = from;  //next number to consider

	    while(nextCand < to) {
		if(isPrime(nextCand)) {
		    pCount++;
		}
		nextCand += 4;
	    }

	    synchronized(lock) {
		sharedPCount += pCount;
	    }
	}
    }

    public static boolean isPrime(long num) {
	//returns whether num is prime

	int limit = (int) Math.sqrt(num);
	for(long i=2; i<=limit; i++) {
	    if(num % i == 0)
		return false;
	}
	return true;
    }

    public static void main(String args[]) throws InterruptedException {
	sharedPCount = 1;   //number of primes found (starting with 2)

	PrimeFinder p1 = new PrimeFinder(3, 2000000);
	Thread t1 = new Thread(p1);
	PrimeFinder p2 = new PrimeFinder(5, 2000000);
	Thread t2 = new Thread(p2);

	t1.start();
	t2.start();

	t1.join();
	t2.join();
	
	System.out.println(sharedPCount + " primes found");
    }
}
